#!/usr/bin/env node

/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/
'use strict';

require('./lib/utils/debugger.js');

const debuglog = __debug.bind(null, __filename);

var pjson = require('./package.json');
var help = require('./lib/cli/help');
var fileUtil = require('./lib/utils/file-util');
var errorHandler = require('./lib/utils/err');
var _ = require('underscore');
global.pjson = pjson;
var validationConst = require('./lib/validations/constants').validationContants;

if (Number(process.versions.node.split(".")[0]) < 4) {
  console.log('Node version of 4.x.x and above is required to run SDK. ' + process.versions.node + ' found.');
  process.exit(0);
}
// Cli Parsing:

// 1. Registering Cli commands
var Program = require('wiz-cliparse');
var prg = new Program('fdk',
  'Freshworks Development Kit.',
  '[command] [command-options] [arguments]');

var cmdInit = prg.addCmd('create',
  '# Create a new app.',
  '[folder]',
  'When [folder] is not given, CWD (if empty) is used to init.');
cmdInit.addOpt('d', 'app-dir', 'App directory.', {
  hasArg: true
});
cmdInit.addOpt('p', 'products', 'Products', {
  hasArg: true
});
cmdInit.addOpt('t', 'template', 'Template to base the project on.', {
  hasArg: true
});
cmdInit.addOpt('h', 'help', 'create command help.');

var cmdRun = prg.addCmd('run', '# Run the app on a local server for testing');
cmdRun.addOpt('d', 'app-dir', 'App directory.', {
  hasArg: true
});
cmdRun.addOpt('c', 'clear-coverage', 'clear code-coverage stats');
cmdRun.addOpt('s', 'skip-coverage', 'skip instrumenting code');
cmdRun.addOpt('h', 'help', 'run command help.');

var cmdValidate = prg.addCmd('validate', '# Run validation tests on the app');
cmdValidate.addOpt('d', 'app-dir', 'App directory.', {
  hasArg: true
});
cmdValidate.addOpt('h', 'help', 'run command help.');

var cmdPack = prg.addCmd('pack', '# Package the app into a zip file');
cmdPack.addOpt('d', 'app-dir', 'App directory.', {
  hasArg: true
});
cmdPack.addOpt('h', 'help', 'run command help.');

var cmdVersion = prg.addCmd('version', '# Display installed and latest version');
cmdVersion.addOpt('d', 'app-dir', 'App directory.', {
  hasArg: true
});
cmdVersion.addOpt('h', 'help', 'run command help.');

// 2. Parse cli options:
var res = null;
try {
  res = prg.parse();
}
catch (err) {
  if (err.startsWith("Unrecognized command")) {
    var cmd = err.match(/Unrecognized command: (.*)./)[1];
    console.error(`'${cmd}' is not a valid command. Please type 'fdk' to get the list of supported commands.`);
  }
  else {
    console.error(err);
  }
  process.exit(1);
}

debuglog(`Running with args ${JSON.stringify(res)}`);

var showHelp = function(command) {
  if (res.opts.has('h')) {
    help.printHelp(command);
    process.exit(0);
  }
}

if (res.opts.has('app-dir')) {
  var directory = res.optArg.get('app-dir');

  debuglog(`Looking for app-dir "${directory}"`);

  if (fileUtil.fileExists(directory)) {
    process.chdir(directory);
  }
  else {
    console.log('The specified path to the directory is not valid.');
    process.exit(1);
  }
}

// Other commands:
switch (res.cmd) {

  case 'create':
    showHelp('create')
    var template = res.opts.has('t') ? res.optArg.get('t') : undefined;
    var product = res.opts.has('p') ? res.optArg.get('p') : undefined;

    require('./lib/cli/init').run(process.cwd(), template, product);
    break;

  case 'run':
    showHelp('run');
    require('./lib/cli/run').run(res.opts.has('c'), res.opts.has('s'));
    break;

  case 'validate':
    showHelp('validate');
    var validationMessages = require('./lib/cli/validate').run(
      validationConst.PRE_PKG_VALIDATION);
    errorHandler.printError('Validation failed due to the following issue(s):', validationMessages);
    break;

  case 'pack':
    showHelp('pack');
    require('./lib/cli/pack').run();
    break;

  case 'version':
    showHelp('version');
    require('./lib/cli/version').run();
    break;

  default:
    prg.printHelp(res);
    console.log(`%sType “fdk COMMAND (-h | --help)” for details on a specific command\n`, Array(3).join(" "));
}
