'use strict'

const debuglog = __debug.bind(null, __filename);

var request = require('request');
var _ = require("underscore");
var validator = require('validator');
var oauth = require("../routes/oauth2");
var manifest = require("../manifest");
var url = require("url");
var configUtil = require("../utils/config-util");
var NETWORK_ERROR_STATUS = 502;
var NETWORK_ERROR_MSG = "Error in establishing connection";
var BAD_WHITELISTED_DOMAIN_STATUS = 400;
var BAD_WHITELISTED_DOMAIN_MSG = "Error while substituting the templates";
var INVALID_CONTENT_STATUS = 415;
var INVALID_CONTENT_MSG = "Unsupported content type";
const OK_STATUS = 200;
const SUPPORTED_ENCODINGS = ['base64'];
const DEFAULT_ENCODING = 'base64';
const VALID_CONTENT_TYPES = ["application/json", "application/xml", "text/html", "text/xml", "application/jsonp", "text/plain"];

var utility = {
  encode: function(data, encoding) {
    encoding = _.contains(SUPPORTED_ENCODINGS, encoding) ? encoding : DEFAULT_ENCODING;
    return new Buffer(data).toString(encoding);
  }
}

function isValidContentType(response) {
  var contentType = (response.headers['content-type']) ? response.headers['content-type'].split(';')[0] : null;

  debuglog(`Looking for "${contentType}" in ${VALID_CONTENT_TYPES}`);

  
  if (contentType && !_.contains(VALID_CONTENT_TYPES, contentType)) {
    return false;
  }

  return true;
}

module.exports = {
  execute: function(req, res) {
    var templates = {};
    templates["iparam"] = configUtil.getValuesForLocalTesting();
    var reqOptions = req.body.data;
    if (_.contains(manifest.features, "oauth") && req.body.data.isOAuth) {
      var oauthTokens = fetchToken();

      templates = _.extend(templates, oauthTokens);
      debuglog(`Extending template values with oauth tokens ${JSON.stringify(oauthTokens)}`);
    }
    try {
      reqOptions = substituteTemplates(reqOptions, _.extend({encode: utility.encode}, templates));
      var urlObject = url.parse(reqOptions.url);
      var domain = `${urlObject.protocol}//${urlObject.hostname}`;
      if(!isValidDomain(substituteTemplates(manifest.whitelistedDomains, templates), domain)) {
        return res.status(OK_STATUS).send({ status: 400, headers: {}, response: "This domain has not been whitelisted", errorSource: APP_SOURCE });
      }
    } catch(err) {
      debuglog(`Proxy errored with ${err.message}`);
      return res.status(OK_STATUS).send({ status: BAD_WHITELISTED_DOMAIN_STATUS, headers: {}, response: BAD_WHITELISTED_DOMAIN_MSG, errorSource: APP_SOURCE });
    }

    debuglog(`Making proxy call with options as ${JSON.stringify(reqOptions)}`);

    request(reqOptions, function(error, response, responseBody) {
      debuglog(`Proxy came back with error as ${error && error.message}, body as ${JSON.stringify(responseBody)}`);

      if (error) {
        return res.status(OK_STATUS).send({ status: NETWORK_ERROR_STATUS, headers: {}, response: NETWORK_ERROR_MSG, errorSource: APP_SOURCE });
      }

      if (!isValidContentType(response)) {
        return res.status(OK_STATUS).send({ status: INVALID_CONTENT_STATUS, headers: {}, response: INVALID_CONTENT_MSG, errorSource: APP_SOURCE });
      }

      var responsePayLoad = { status:response.statusCode, headers:response.headers, response:responseBody };

      if (response.statusCode >= 400) {
        _.extend(responsePayLoad, {errorSource: APP_SOURCE});
      }

      res.status(OK_STATUS).send(responsePayLoad);
    });
  }
};

function isValidDomain(whitelistedDomains, endpoint) {
  const urlInfo = url.parse(endpoint);

  debuglog(`Parsed ${endpoint} as ${JSON.stringify(urlInfo)}`);

  if(urlInfo.protocol == 'https:' && validator.isFQDN(urlInfo.hostname)) {
    return isWhitelistedDomain(whitelistedDomains, endpoint);
  }

  debuglog(`Domain ${endpoint} is not valid.`);

  return false;
}

function fetchToken() {
  return { 'access_token' : oauth.fetchCredentials()['access_token'] };
}

function substituteTemplates(options, templates) {
  debuglog(`Substituing Templates ${JSON.stringify(options)} with ${JSON.stringify(templates)}`);

  return JSON.parse(_.template(JSON.stringify(options))(templates));
}

function isWhitelistedDomain(whitelistedDomains, url) {
  for(var domain of whitelistedDomains) {
    if(domain.startsWith('https://*')) {
      domain = domain.replace('*', '[a-z0-9\-]*');
      var exp = new RegExp(domain);
      if(url.match(exp)) {
        debuglog(`Matched "${url}" with ${domain}`);
        return true;
      }
    }
    else if(domain === url) {
      debuglog(`Matched "${url}" with ${domain}`);
      return true;
    }
  }

  return false;
}
