/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/

'use strict';

const debuglog = __debug.bind(null, __filename);

var fs = require('fs-extra');
var _ = require('underscore');
var errorHandler = require('../utils/err');
var nodeTree = require('nodetree');
var inquirer = require('inquirer');
var util = require("../utils/helper");

var RE_IGNORED_FILES = /^\.(.)*$|^\.Spotlight-V100$|\.Trashes$|^Thumbs\.db$|^desktop\.ini$/g;

const TEMPLATE_ORDER = {
  freshdesk: [
    'your_first_app',
    'your_first_serverless_app',
    'sample_crm_app'
  ],
  freshservice: [
    'your_first_app',
    'your_first_serverless_app'
  ],
  freshsales: [
    'your_first_serverless_app'
  ]
};

const SERVERLESS_APPS = {
  'freshdesk': ['your_first_serverless_app'],
  'freshsales': ['your_first_serverless_app'],
  'freshservice': ['your_first_serverless_app']
};

function getTemplatesFor(product) {
  return _.intersection(TEMPLATE_ORDER[product], fs.readdirSync(__dirname + `/../../template/${product}`).filter(e => !e.match(RE_IGNORED_FILES)));
}

const templatesAllowed = {
  freshdesk: getTemplatesFor('freshdesk'),
  freshservice: getTemplatesFor('freshservice'),
  freshsales: getTemplatesFor('freshsales')
};
const MINIMUM_TEMPLATE_COUNT = 1;

const supportedProducts = Object.keys(templatesAllowed);

module.exports = {
  run: function(prjDir, template, product) {
    debuglog(`'init' called with ${JSON.stringify(arguments)}`);

    var errMessages = [];

    if (!product) {
      product = 'freshdesk';
      debuglog(`Assuming default product ${product}`);
    }

    if (!_.contains(supportedProducts, product)) {
      errMessages.push(`The specified product ${product} is not valid. The supported products are ${supportedProducts}.`);
    }
    else {
      if (template && !_.contains(templatesAllowed[product], template)) {
        errMessages.push(`The specified template ${template} is not valid. The supported templates are ${templatesAllowed[product]}.`);
      }
    }
    if (!_.isEmpty(fs.readdirSync(prjDir))) {
      errMessages.push('The current directory is not empty. Run the command inside an empty directory or use the --app-dir option to specify the path to a new or empty directory.');
    }

    errorHandler.printError('The app could not be created due to the following issues(s):', errMessages);

    if (template) {
      // Copy from project template:
      initTemplate(product, template, prjDir);
    }
    else if (!template && templatesAllowed[product].length > MINIMUM_TEMPLATE_COUNT) {
      return inquirer.prompt({
        type: 'list',
        name: 'template',
        message: 'Choose a template:',
        choices: _.intersection(TEMPLATE_ORDER[product], templatesAllowed[product])
      }).then(val => {
        initTemplate(product, val.template, prjDir );
      });
    }
    else {
      initTemplate(product, templatesAllowed[product][0], prjDir);
    }
  }
};

function initTemplate(product, template, prjDir){
  debuglog(`Initializing template "${template}" with product "${product}".`);

  fs.copySync(__dirname + `/../../template/${product}/${template}`, prjDir);

  if (_.contains(SERVERLESS_APPS[product], template)) {
    fs.copySync(__dirname + `/../../config/payloads/${product}/`, `${prjDir}/server/test_data`);
  }
  console.log(`A new ${util.capitalize(product)} app was successfully created from template "${template}" with the following files.\n`);
  nodeTree(process.cwd());
}
