/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/
'use strict';

const debuglog = __debug.bind(null, __filename);

var _ = require('underscore');
var fs = require('fs-extra');
var archiver = require('archiver');
var eh = require('../utils/err');
var nsUtil = require('../utils/ns-resolver');
var fileUtil = require('../utils/file-util');
var validator = require('./validate');
var digestFile = require('../utils/digest-file-handler');
var validationConst = require('../validations/constants').validationContants;
var async = require('async');
var nodeDir = require('node-dir');
var zip = archiver('zip');
var pkgErrorMsg = "Error while generating package.";
var cwd = process.cwd();
var pkgName = `${nsUtil.getRootFolder()}${nsUtil.pkgExt}`;
var manifest = require('../manifest');
var util = require('../utils/helper');
var path = require('path');

var SERVER_DIR_ALLOWED_FILES = /.js$/;
var SERVER_DIR_EXCLUDED_FOLDERS = ['node_modules', 'etc', 'test_data'];
var DOCUMENTATION_REFERENCE = {
  freshdesk: 'https://developer.freshdesk.com/v2/docs/freshdesk-apps/',
  freshsales: 'https://developer.freshdesk.com/v2/docs/freshdesk-apps/',
  freshservice: 'https://developer.freshservice.com/docs/freshservice-apps/'
};
function addFilesToZip() {
  async.series([
    function(callback) {
      if (!fs.existsSync('./app')) {
        debuglog('Cannot find app folder while packing.');
        return callback();
      }
      fileUtil.getFiles({dir: './app'}, function(err, files) {
        if (err) {
          eh.error(new Error(pkgErrorMsg));
        }
        for (var file of files) {
          debuglog(`Packing file ./app/${file}`);
          zip.file(`./${file}`)
        }
        callback(null);
      });
    },
    function(callback) {
      if (!_.contains(manifest.features, 'backend')) {
        debuglog('Not a backend app, skipped looking for server folder.');
        return callback();
      }
      async.each(_.difference(fs.readdirSync('./server'), SERVER_DIR_EXCLUDED_FOLDERS), function(file, cb) {
        var serverChild = `./server/${file}`;
        if(fs.lstatSync(serverChild).isDirectory()) {
          fileUtil.getFiles({dir: serverChild, match: SERVER_DIR_ALLOWED_FILES}, function(err, files) {
            if (err) {
              eh.error(new Error(pkgErrorMsg));
            }
            for (var file of files) {
              debuglog(`Packing file ${serverChild}/${file}`);
              zip.file(`./${file}`);
            }
            cb();
          });
        }
        else {
          debuglog(`Packing file ${serverChild}`);
          zip.file(serverChild)
          cb();
        }
      }, function() {
        callback();
      });
    },
    function(callback) {
      fileUtil.getFiles({dir: './config'}, function(err, files) {
        if (err) {
          eh.error(new Error(pkgErrorMsg));
        }
        for (var file of files) {
          debuglog(`Packing file config/${file}.`);
          zip.file(`./${file}`)
        }
        callback(null);
      });
    },
    function(callback) {
      zip.file('./manifest.json');
      debuglog('Packing manifest.');
      callback(null);
    },
    function(callback) {
      if(fileUtil.fileExists('./.report.json')) {
        zip.file('./.report.json');
        debuglog('Packing report.');
      }
      callback(null);
    },
    function(callback) {
      digestFile.genDigestFile(function() {
        zip.file('./digest.md5');
        debuglog('Packing digest.');
        callback(null);
      });
    }
  ], function() {
    debuglog('Finalizing zip.');
    zip.finalize();
  })
}

function zipPackage(output) {
  output.on('close', function(err) {
    if (err) {
      cleanFiles(true);
      eh.error(new Error("Error while creating package."));
    }

    var postPkgValidation = validator.run(validationConst.POST_PKG_VALIDATION);
    var product = _.keys(manifest.product)[0];
    eh.printError('Packing failed due to the following issue(s):', postPkgValidation);
    console.log(`App packed successfully at ${path.sep}dist${path.sep}${pkgName}`);
    console.log(`Upload this file to the marketplace by following the instructions at ${DOCUMENTATION_REFERENCE[product]}`);
    cleanFiles();
  });
  zip.on('error', function() {
    cleanFiles(true);
    eh.error(new Error(pkgErrorMsg));
  });
  zip.pipe(output);
  addFilesToZip(zip);
}

function cleanFiles(err) {
  digestFile.delDigestFile();

  if (err) {
    debuglog('Deleting dist/ folder.');
    fileUtil.deleteFile(cwd + "/dist");
  }
}

function packApp() {
  var dest = cwd + `/dist/${pkgName}`;
  fileUtil.ensureFile(dest);
  var output = fs.createWriteStream(dest);
  zipPackage(output);
}

module.exports = {
  run: function() {
    debuglog(`'pack' called with ${JSON.stringify(arguments)}`);

    var errors = validator.run(validationConst.PRE_PKG_VALIDATION);

    if(errors.length > 0) {
      eh.printError('Packing failed due to the following issue(s):', errors);
    }

    try {
      packApp();
    } catch(e) {
      eh.printError('Some exception happened during pack - ', e);
    }
  }
};
