/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/

'use strict';

const debuglog = __debug.bind(null, __filename);

var dpRouter = require('../routes/data-pipe');
var iframe = require('../routes/iframe');
var configs = require('../web/routes/configs.js');
var beEvents = require('../routes/beevents');
var webEvents = require('../web/routes/events.js');
var oauth2 = require('../routes/oauth2');
var validator = require('./validate');
var validationConst = require('../validations/constants').validationContants;
var manifest = require('../manifest');
var eh = require('../utils/err');
var httpUtil = require("../utils/http-util");
var fileUtil = require('../utils/file-util');
var dependencyInstaller = require('../utils/install');
var watcher = require('../watcher');
var _ = require('underscore');
var fs = require('fs');
var HTTP_PORT = 10001;
var util = require("../utils/helper");
var express = require('express');
var ws = require('express-ws');
var app = express();
// Register middleware:
var bodyParser = require('body-parser');
var expressWs = ws(app);
const DOMAIN = {
  freshdesk: 'https://domain.freshdesk.com/a/tickets/1?dev=true',
  freshservice: 'https://domain.freshservice.com/helpdesk/tickets/1?dev=true',
  freshsales: 'https://domain.freshsales.io/contacts/2007663673?dev=true'
};

const coverageUtil = require('../utils/coverage.js');

const WSS_SCRIPT = `
  <script>
    var ws = new WebSocket("ws://localhost:10001/notify-change");
    ws.onmessage = function(e) {
      location.reload();
    };
  </script>
`;

global.PLATFORM_SOURCE = 'PLATFORM';
global.APP_SOURCE = 'APP';

function logServerStartMsg() {
  var product = _.keys(manifest.product)[0];
  console.log(`Starting local testing server at http://*:${HTTP_PORT}/\n`
    + `Append 'dev=true' to your ${util.capitalize(product)} account URL to start testing\n`
    + `e.g. ${DOMAIN[product]}\n`
    + `Quit the server with Control-C.`);
  if (_.contains(manifest.features, 'backend')) {
    console.log('\nTo simulate product, app setup, and external events, visit - http://localhost:10001/web/events');
  }
  if (fileUtil.fileExists(`${process.cwd()}/config/iparams.html`)) {
    console.log('To test the custom installation page, visit - http://localhost:10001/custom_configs')
  }
}

module.exports = {
  run: function(clearCoverage, skipCoverage) {
    var validationMessages = validator.run(validationConst.RUN_VALIDATION);
    eh.printError('The local server could not be started due to the following issue(s):', validationMessages);

    if (clearCoverage) {
      coverageUtil.dispose();
    }

    if (skipCoverage) {
      coverageUtil.snooze();
    }

    var exWss = expressWs.getWss('/notify-change');

    /**
     *  Istanbul might send back massive coverage reports as JSON.
     *  So have a body-parser with higher limits for that route 
     *  alone.
     */
    const istanbulBodyParser = bodyParser.json({
      limit: '5MB'
    });
    
    const normalBodyParser = bodyParser.json();
    
    app.use(function(req, res, next) {
      if (req.path === '/iframe/coverage') {
        return istanbulBodyParser(req, res, next);
      }
      
      return normalBodyParser(req, res, next);
    });

    /*
    IFrame:
    All the contents inside the app directory being served as static files.
    */
    app.use(function (req, res, next) {
      httpUtil.enableCORS(res);
      next();
    });
    app.use('/iframe/oauth', express.static(__dirname +'/../web/pages/oauth'));

    const staticMiddleware = express.static('./app');

    app.use('/iframe', function(req, res, next) {
      const fullPath = `./app${req.path}`;

      if (req.method === 'GET') {
        if (req.path.endsWith('.js')) {
          debuglog(`Responding with contents of ${req.path}`);
          return res.send(coverageUtil.instrument(fullPath));
        }

        if (req.path.endsWith('.html')) {
          debuglog(`Responding with contents of ${req.path}`);
          return res.send(fs.readFileSync(fullPath, 'utf8') + WSS_SCRIPT);
        }
      }

      return staticMiddleware(req, res, next);
    });

    app.use('/web/assets', express.static(__dirname + '/../web/assets'))

    app.post('/iframe/coverage', function(req, res) {
      const coverageStats = req.body;

      coverageUtil.update(coverageStats)

      res.send('OK');
    });

    app.ws('/notify-change', function(ws) {
      if (process.env.NODE_ENV === 'test') {
        setTimeout(function() {
          ws.close();
        }, 4000);
      }
    });

    watcher.watch(function(data) {
      exWss.clients.forEach(function(client) {
        client.send(data);
      });
    });

    dpRouter.run(app);
    iframe.run(app);
    configs.run(app);
    beEvents.run(app);
    webEvents.run(app);
    oauth2.run(app);

    // Finally, listen:
    var server = app.listen(HTTP_PORT, function() {
      if (_.contains(manifest.features, 'backend')) {
        dependencyInstaller.run(function(err) {
          if (err) {
            console.log(err);
            process.exit(1);
          }
          logServerStartMsg();
        });
      } else {
        logServerStartMsg();
      }
    });

    process.on('uncaughtException', function(err) {
      if (err.code === 'EADDRINUSE') {
        console.error("Another instance of server running? Port in use.");
      }
      else {
        console.error(err);
      }
      process.exit(1);
    });

    return server;
  }
};
