'use strict';

var _ = require('underscore');
var oauth = require("../../routes/oauth2");
var configUtil = require("../../utils/config-util");
var request = require('request');
var jQueryDeferred = require('jquery-deferred');
var manifest = require('../../manifest');

const DP_ROUTER_URL = 'http://localhost:10001/dprouter'
const INTERNAL_SERVER_ERR_STATUS = 500;
const NETWORK_ERROR_STATUS = 502;
const TIMEOUT_ERROR_STATUS = 504;
const INVALID_CONTENT_STATUS = 415;
const INTERNAL_SERVER_ERR_MSG = "Internal server error";
const NETWORK_ERROR_MSG = "Error in establishing connection";
const TIMEOUT_ERR_MSG = "Timeout error while processing the request";
const INVALID_CONTENT_MSG = "Unsupported content type";
const VALID_CONTENT_TYPES = ["application/json", "application/xml", "text/html", "text/xml", "application/jsonp", "text/plain"];
const APP_SOURCE = 'APP';
const PLATFORM_SOURCE = 'PLATFORM';
const TIMEOUT_RESP = { status: TIMEOUT_ERROR_STATUS, headers: {}, response: TIMEOUT_ERR_MSG, errorSource: APP_SOURCE };
const NETWORK_ERR_RESP = { status: NETWORK_ERROR_STATUS, headers: {}, response: NETWORK_ERROR_MSG, errorSource: APP_SOURCE };
const INVALID_CONTENT_RESP = { status: INVALID_CONTENT_STATUS, headers: {}, response: INVALID_CONTENT_MSG, errorSource: APP_SOURCE };
const INTERNAL_SERVER_ERR = { status: INTERNAL_SERVER_ERR_STATUS, headers: {}, response: INTERNAL_SERVER_ERR_MSG, errorSource: APP_SOURCE};
const TIMEOUT_CODE = "ETIMEDOUT";
const SOCKET_TIMEOUT = "ESOCKETTIMEDOUT";
const REQUEST_METHODS = ['get', 'post', 'put', 'patch', 'delete'];
const REQUEST_TIMEOUT = 4500;
const REDIRECTS_FOLLOW = false;
const SUPPORTED_ENCODINGS = ['base64'];
const DEFAULT_ENCODING = 'base64';

var utility = {
  encode: function(data, encoding) {
    encoding = _.contains(SUPPORTED_ENCODINGS, encoding) ? encoding: DEFAULT_ENCODING;
    return new Buffer(data).toString(encoding);
  }
}

class RequestApi {
  constructor(args) {
    var self = this;
    REQUEST_METHODS.forEach(function(method) {
      self[method] = function(url, options) {
        return new Request(args).makeRequest(method, url, options);
      }
    });
  }
}

class Request {
  constructor(args) {
    this.requestDefer = jQueryDeferred.Deferred();
  }

  makeRequest(action, url, options, recursive) {
    var self = this;
    if (_.contains(manifest.features, "oauth")) {
      self.access_token = oauth.fetchCredentials()['access_token'] ? oauth.fetchCredentials()['access_token'] : '';
    }
    var params = _.extend({
      url: url,
      method: action,
      followAllRedirects: REDIRECTS_FOLLOW,
      timeout: REQUEST_TIMEOUT
    }, options);
    var modifiedParams = JSON.parse(_.template(JSON.stringify(params))({
      iparam: configUtil.getValuesForLocalTesting(),
      access_token: self.access_token,
      encode: utility.encode
    }));
    request(modifiedParams, function(err, response, body) {
      if (err) {
        let errResp = (err.code === TIMEOUT_CODE || err.code === SOCKET_TIMEOUT) ? TIMEOUT_RESP : NETWORK_ERR_RESP;
        self.requestDefer.reject(errResp);
      }
      else {
        var contentType = (response.headers['content-type']) ? response.headers['content-type'].split(';')[0] : null;
        if (response.statusCode === 401 && params.isOAuth && !recursive) {
          var dpRouterOptions = _.extend({}, self.reqOptions);
          request({
            method: "POST",
            url: DP_ROUTER_URL,
            headers: { 'mkp-route' : 'oauth'},
            json: { action: 'refresh'}
          }, function(err, resp, body) {
            if (err) {
              return self.requestDefer.reject(err);
            }
            else {
              if (body && (body.status == 400 || body.status == 210)) {
                return self.requestDefer.reject(_.extend(body, {errorSource: PLATFORM_SOURCE}));
              }
              return self.makeRequest(action, url, options, true);
            }
          });
        }
        else if(contentType && !_.contains(VALID_CONTENT_TYPES, contentType)) {
          return self.requestDefer.reject(INVALID_CONTENT_RESP);
        }
        else {
          var proxyResponse = {status: response.statusCode, headers: response.headers, response: body};
          if (response.statusCode >= 400) {
            _.extend(proxyResponse, { errorSource: APP_SOURCE });
            return self.requestDefer.reject(proxyResponse);
          }
          self.requestDefer.resolve(proxyResponse);
        }
      }
    });
    return self.requestDefer.promise();
  }
}

module.exports = RequestApi;
