'use strict';
var fs = require("fs");
var vm = require("vm");
var _ = require('underscore');
var serverScriptPath = `${process.cwd()}/server/server.js`;
var uuid = require('uuid');
var path = require("path");
var manifest = require('../manifest');
var RequestAPI = require('./features/request');
var ScheduleAPI = require('./features/schedule');
var INTERNAL_ERROR_STATUS = 500;
var INVALID_DATA_ERR_STATUS = 400;
var TIMEOUT_ERR_STATUS = 504;
var OK_STATUS = 200;
var INVALID_DATA_ERR_MSG = 'The error should be a JSON Object with a message or a status parameter.'
var TIMEOUT_ERR_MSG = 'Timeout error while processing the request.';
var FW_ERROR_MSG = 'Error while executing App server script!';
var FEATURE_DEFINITIONS = {
  "db" : "$db"
}
var util = require('util');

const nestedPath = [ `${process.cwd()}/server/` ];

const coverageUtil = require('../utils/coverage');

function createModuleError(modulePath) {
  var error = new Error(`Cannot find module "${modulePath}"`);
  error.code = 'MODULE_NOT_FOUND';
  return error;
}

function loadDependency(dpn) {
  dpn = path.basename(dpn);
  return require(`${process.cwd()}/server/node_modules/${dpn}`);
}

const APP_EVENT_TIMEOUT = 10000,
  PRODUCT_EVENT_TIMEOUT = 20000,
  EXTERNAL_EVENT_TIMEOUT = 20000,
  SMI_TIMEOUT = 5000;

class Framework {

  constructor(event, req, res) {
    var self = this;

    this.event = event;
    this.req = req;
    this.res = res;

    this.cachedModules = {}

    this.serverScript = coverageUtil.instrument(serverScriptPath);
    /**
      We are running the developers code in the sandbox.
      By running in sandbox, we allow the user to only use certain methods.
      We use require to load the node modules [in loadDependency] as it contains the methods, which are outside of the context.
      We load the local library files[ loadLib ] by reading the files, restricting the user from using only the methods passed to the context.
    */
    this.sandboxAPI = _.extend({
      __fdkcoverage__: {},
      exports: module.exports,
      console: {
        log: console.log,
        info: console.info,
        error: console.error
      },
      loadLib: util.deprecate(function loadLib(fileName) {
        fileName = path.basename(fileName);
        var libScript = coverageUtil.instrument(`${process.cwd()}/server/lib/${fileName}.js`);
        return self.sandboxExecutor(libScript);
      }, '`loadLib` will be deprecated soon. Please use `require`.'),
      loadDependency: util.deprecate(loadDependency, '`loadDependency` will be deprecated soon. Please use `require`.'),
      require: function (relativePath) {
        var script,
            exported,
            basePath,
            fullPath;

        if (relativePath.startsWith('./') || relativePath.startsWith('../')) {
          basePath = path.resolve.apply(null, nestedPath);
          fullPath = path.normalize(`${basePath}/${relativePath}.js`);

          if (self.cachedModules.hasOwnProperty(fullPath)) {
            return self.cachedModules[fullPath];
          }

          if (!fullPath.startsWith(path.resolve(`${process.cwd()}/server/`))) {
            throw createModuleError(relativePath);
          }

          if (!fs.existsSync(fullPath)) {
            throw createModuleError(relativePath);
          }

          script = coverageUtil.instrument(fullPath);

          nestedPath.push(path.dirname(relativePath));
          exported = self.sandboxExecutor(script);

          nestedPath.pop();
          self.cachedModules[fullPath] = exported;

          return exported;
        }

        return loadDependency(relativePath);
      }
    }, getFeatures(self, req));

    (function() {
      var previous;

      setInterval(function() {
        var current = JSON.stringify(self.sandboxAPI.__fdkcoverage__);

        if (previous !== current) {
          coverageUtil.update(self.sandboxAPI.__fdkcoverage__);
          previous = current;
        }
      }, 2000).unref();
    })();

    // When the developer doesn't use renderData then we don't return the requestID in response. It would be an empty success response.
    _.extend(this.sandboxAPI, {
      "renderData" : function (error, output) {
        try {
          var returnVal = null;
          var statusCode = 200;
          var requestID = self.req.RequestId;
          var showInUI = false;

          if (error) {
            var response = {};
            if (_.isArray(error) || !(_.isObject(error)) || (_.isUndefined(error.message) && !error.status)) {
              statusCode = INVALID_DATA_ERR_STATUS;
              response = { "status" : statusCode, "message" : INVALID_DATA_ERR_MSG, "errorSource": APP_SOURCE };
            }
            else {
              if (_.isUndefined(error.message)) {
                error.message = null;
              }
              statusCode = error.status || INTERNAL_ERROR_STATUS;
              response = { "status" : statusCode, "message" : error.message, "errorSource": APP_SOURCE };
              showInUI = true;
            }
            returnVal = _.extend({"requestID" : requestID}, response );
          }
          else {
            returnVal = { "requestID" : requestID };
            if (!_.isUndefined(output)) {
              returnVal = _.extend({"requestID" : requestID}, {"response" : output});
            }
          }

          self.res.header('showinui', showInUI);
          self.res.status(statusCode).json(returnVal);
        }
        catch (e) {
          console.log(e);
        }
      }
    });
    this.sandboxAPI.global = this.sandboxAPI;
    this.contextifiedGlobal = new vm.createContext(this.sandboxAPI);
  }

  getMethodName() {
    return this.event['categoryArgs']['methodName'];
  }

  getMethodArgs() {
    return this.event['categoryArgs']['methodParams'];
  }

  execute(methodName, args) {
    var jsonArgs = JSON.stringify(args || {});
    var executorScript = `__svrScript['${methodName}'](${jsonArgs})`;
    _.extend(this.contextifiedGlobal, { __svrScript: this.svrExecScript });
    this.sandboxExecutor(executorScript);
  }

  sandboxExecutor(script) {
    var serverScript = this.constructScript(script);
    return serverScript.runInContext(this.contextifiedGlobal, { timeout: this.event.timeout });
  }

  constructScript(script) {
    script = `(function() {var exports = {}; ${script}; return exports;})()`;
    return new vm.Script(script, {
      "filename": "server.js"
    });
  }

  handleError(err) {
    console.log(err);
    var output = {
      "requestID" : this.req.RequestId,
      "status" : INTERNAL_ERROR_STATUS,
      "message" : `${FW_ERROR_MSG} - ${err.message}`,
      "errorSource": APP_SOURCE
    };
    this.res.status(OK_STATUS).json(output);
  }

}

class Request extends Framework {

  constructor(event, req, res) {
    event.timeout = event.timeout || SMI_TIMEOUT;
    super(event, req, res);
  }

  execute() {
    try {
      var self = this;
      setTimeout( function() {
        if (!self.res._headerSent) {
          self.res.status(OK_STATUS).send({ status: TIMEOUT_ERR_STATUS, message: TIMEOUT_ERR_MSG, errorSource: APP_SOURCE });
        }
      }, 5000);
      super.execute(this.getMethodName(), this.getMethodArgs());
    }
    catch (err) {
      this.handleError(err);
    }
  }

}

class Event extends Framework {

  constructor(event, req, res) {
    super(event, req, res);
  }

  getMethodName() {
    var methodName = super.getMethodName();
    var events = this.svrExecScript.events || [];
    for (var i in events) {
      if (events[i]['event'] === methodName) {
        methodName = events[i]['callback'];
        break;
      }
    }
    return methodName;
  }

  execute() {
    try {
      super.execute(this.getMethodName(), this.getMethodArgs());
    }
    catch (err) {
      this.handleError(err);
    }
  }
}

class ProductEvent extends Event {
  constructor(event, req, res) {
    event.timeout = event.timeout || PRODUCT_EVENT_TIMEOUT;
    super(event, req, res);
  }
}

class ExternalEvent extends Event {
  constructor(event, req, res) {
    event.timeout = event.timeout || EXTERNAL_EVENT_TIMEOUT;
    super(event, req, res);
  }
}

class AppEvent extends Event {
  constructor(event, req, res) {
    event.timeout = event.timeout || APP_EVENT_TIMEOUT;
    super(event, req, res);
  }
}

var handler  = function (req, res) {
  req.RequestId = `${uuid.v4()}`;
  if (req.body.categoryName) {
    var obj = fetchEventObj(req, res);
    try {
      obj.svrExecScript = obj.sandboxExecutor(obj.serverScript);
      obj.execute();
    } catch(err) {
      console.log(err);
      var output = {
        "requestID" : req.RequestId,
        "status" : INTERNAL_ERROR_STATUS,
        "message" : `${FW_ERROR_MSG} - ${err.message}`,
        "errorSource": APP_SOURCE
      };
      res.status(OK_STATUS).json(output);
    }
  }
  else {
    var output = {
      requestID: req.RequestId,
      status: INTERNAL_ERROR_STATUS,
      message: FW_ERROR_MSG,
      errorSource: PLATFORM_SOURCE
    }
    res.status(OK_STATUS).json(output);
  }
};

function getFeatures(self, req) {
  var _self = self;
  var enabledFeatures = {};
  _.map(_.intersection(manifest.features, _.keys(FEATURE_DEFINITIONS)), function(featureName) {
    enabledFeatures[FEATURE_DEFINITIONS[featureName]] = require(`./features/${featureName}`);
  });
  enabledFeatures['generateTargetUrl'] = function(options) {
    return require('./features/generateTargetUrl')(_self, options);
  }
  enabledFeatures['$request'] = new RequestAPI;
  enabledFeatures['$schedule'] = new ScheduleAPI(_self);
  return enabledFeatures;
}

function fetchEventObj(req, res) {
  var event = req.body;
  switch (req.body.categoryName) {
    case 'request':
      return new Request(event, req, res);
    case 'appEvent':
      return new AppEvent(event, req, res);
    case 'productEvent':
      return new ProductEvent(event, req, res);
    case 'externalEvent':
      return new ExternalEvent(event, req, res);
  }
}

module.exports = handler;
