'use strict';

var fs = require('fs');
var passport = require('passport');
var OAuth2Strategy = require('passport-oauth2').Strategy;
var nsUtil = require('../utils/ns-resolver');
var storage = require('../utils/data-store');
var httpUtil = require("../utils/http-util");
var configUtil = require('../utils/config-util');
var fileUtil = require('../utils/file-util');
var _ = require('underscore');
var refresh = require('passport-oauth2-refresh');
var uuid = require('uuid');
var STRATEGY = 'oauth2';
var callback = '';
var oauthIparams = {};
var charset = 'utf8';
const BLANK_STATE = "State is blank";
const INVALID_STATE = "Invalid state";

var storeCredentials = function(data) {
  try {
    storage.store(keyNamespace(), data);
  }
  catch (err) {
    return err;
  }
}

var fetchCredentials = function() {
  try {
    return storage.fetch(keyNamespace());
  }
  catch (err) {
    return err;
  }
}

var keyNamespace = function() {
  return `${nsUtil.getNamespace()['app_id']}_oauth`;
}

var oauthInit = function(req, res, next) {
  if (req.query.oauth_iparams) {
    oauthIparams = JSON.parse(req.query.oauth_iparams);
  }
  var oauthConfig = JSON.parse(fs.readFileSync(
    './config/oauth_config.json', charset));
  var oauthStrategy = new OAuth2Strategy({
      clientID: oauthConfig['client_id'],
      clientSecret: oauthConfig['client_secret'],
      callbackURL: 'http://localhost:10001/auth/callback',
      authorizationURL: _.template(oauthConfig['authorize_url'])({oauth_iparams: oauthIparams}),
      tokenURL: _.template(oauthConfig['token_url'])({oauth_iparams: oauthIparams})
    },
    function(accessToken, refreshToken, profile, done) {
      let credentials = {
        'access_token': accessToken
      };
      if (refreshToken) {
        _.extend(credentials, {
          'refresh_token': refreshToken
        })
      }
      storeCredentials(credentials);
      res.writeHead(302, {
        'Location': callback
      });
      res.end();
    }.bind({
      'response': res
    })
  );
  if (!(_.isEmpty(oauthConfig['options']))) {
    oauthStrategy.authorizationParams = function() {
      return oauthConfig['options'];
    };
  }
  passport.use(oauthStrategy);
  refresh.use(oauthStrategy);
  if (next) {
    next();
  }
}

var refreshInit = function(req, res) {
  var data = fetchCredentials();
  if (data['refresh_token']) {
    refresh.requestNewAccessToken(STRATEGY, data['refresh_token'], function(
      err, accessToken, refreshToken) {
      if (accessToken || refreshToken) {
        let credentials = {
          'access_token': accessToken || data['access_token']
        };
        _.extend(credentials, {
          'refresh_token': refreshToken || data['refresh_token']
        })
        storeCredentials(credentials);
        res.status(200).json({ success: true });
      }
      else if(err){
        let credentials = {
          'access_token': data['access_token']
        };
        _.extend(credentials, {
          'refresh_token': data['refresh_token']
        })
        storeCredentials(credentials);
        res.status(err.statusCode).send({ status: err.statusCode, headers: {}, response: err.data });
      }
    });
  }
  else {
    var responsePayLoad = { status: 210, headers: {}, response: "Access token cannot be refreshed" };
    res.status(210).send(responsePayLoad);
  }
}

var verifyState = function(req, res, next) {
  var state = req.query.state;
  try{
    if(!state){
      throw new Error(BLANK_STATE);
    }
    else{
      if (state != fetchCredentials()['state']) {
        throw new Error(INVALID_STATE);
      }
    }
  } catch(e) {
    return res.end(INVALID_STATE);
  }
  next();
}

module.exports = {

  fetchCredentials: fetchCredentials,

  refresh:function(req, res) {
    oauthInit(req, res);
    refreshInit(req, res);
  },

  run: function(app) {

    app.use('/auth/index', function(req, res, next) {
      if (req.query['callback']) {
        callback = req.query['callback'];
      }
      var oauthIParams = configUtil.getOAuthIparam().oauth_iparams;
      if (oauthIParams && !_.isEmpty(oauthIParams) && !req.query.oauth_bypass) {
        return res.send(fileUtil.readFile(__dirname + '/../web/pages/oauth-forms.html'));
      }
      oauthInit(req, res, next);
    });

    app.use('/auth/callback', verifyState);

    app.use('/auth/callback', function(req, res, next) {
      oauthInit(req, res, next);
    });

    app.use('/accesstoken', function(req, res, next) {
      httpUtil.enableCORS(res);
      next();
    });

    app.get('/auth/index', function(req, res) {
      var stateUuid = uuid.v4();
      storeCredentials({ state: stateUuid });
      passport.authenticate(STRATEGY, {
        state: stateUuid
      })(req, res);
    });

    app.get('/auth/callback', passport.authenticate(STRATEGY, {
      session: false
    }));

    app.get('/accesstoken', function(req, res) {
      httpUtil.enableCORS(res);
      var credential = fetchCredentials();
      if (credential) {
        res.send(credential['access_token']);
      }
      else {
        res.status(404).send();
      }
    });

    app.get('/oauth_iparams', function(req, res) {
      var oauthConfig = configUtil.getOAuthIparam();
      if (oauthConfig.oauth_iparams && !_.isEmpty(oauthConfig.oauth_iparams)) {
        res.json(oauthConfig.oauth_iparams);
      }
      else {
        res.json({});
      }
    });
  }
}
