/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/
'use strict';

const debuglog = __debug.bind(null, __filename);

var _ = require('underscore');
var fs = require('fs-extra');
var eh = require("./err");
var path= require('path');
var nodeDir = require('node-dir');

var RE_IGNORED_FILES = /^\.(.)*$|^\.Spotlight-V100$|\.Trashes$|^Thumbs\.db$|^desktop\.ini$/g;
var DEFAULT_MATCH = /.*/;
var ENCODING = 'utf8';

module.exports = {

  getFiles: function(options, callback) {
    nodeDir.readFiles(options.dir, {
      exclude: RE_IGNORED_FILES,
      match: options.match || DEFAULT_MATCH
    }, function(err, content, next) {
      if (err) { return callback(err); }
      next();
    },
    function(err, files) {
      if (err) { return callback(err); }
      callback(null, files);
    });
  },

  ensureFile: function(fileName) {
    try {
      fs.ensureFileSync(fileName);
    }
    catch (err) {
      eh.error(err);
    }
  },

  writeFile: function(file, data) {
    try {
      fs.writeFileSync(file, data);
    }
    catch (err) {
      eh.error(err);
    }
  },

  deleteFile: function(file) {
    try {
      fs.removeSync(file);
    }
    catch (err) {
      eh.error(err);
    }
  },

  readFile: function(file, encoding) {
    try {
      var charset = encoding || ENCODING;
      return fs.readFileSync(file, charset);
    }
    catch (err) {
      eh.error(err);
    }
  },

  readDir: function(dir) {
    try {
      return fs.readdirSync(dir);
    }
    catch (err) {
      eh.error(err);
    }
  },

  statFile: function(file) {
    try {
      return fs.statSync(file);
    }
    catch (err) {
      eh.error(err);
    }
  },

  fileExists: function(file) {
    try {
      fs.accessSync(file);
    }
    catch (err) {
      return false;
    }
    return true;
  },

  removeJunkFiles: function(file) {
    return !file.match(RE_IGNORED_FILES);
  },

  spider: function(SKIP_FOLDERS, ALLOWED_EXTNS, directory) {
    const markedFiles = [];

    function walk(directory) {
      var files;

      try {
        files = fs.readdirSync(directory);
      }
      catch(e) {
        return;
      }

      for (let i = 0; i < files.length; i++) {
        let file = path.join(directory, files[i]);

        if (fs.statSync(file).isFile()) {
          let fileExt = path.extname(files[i]).toLowerCase();

          if (ALLOWED_EXTNS.includes(fileExt)) {
            markedFiles.push(file)
          }
        } else if (fs.statSync(file).isDirectory() && !SKIP_FOLDERS.includes(files[i])) {
          walk(file)
        }
      }
    }

    walk(directory);

    return markedFiles;
  },

  /**
   *  Try requiring the file. Else, fallback to empty exports.
   */
  safeRequire: function(file) {
    try {
      return require(file);
    } catch(e) {
      debuglog(`Error while requiring ${file}: ${e.message}`);
      return {};
    }
  }
};
