/*
Copyright (C) 2016 Freshdesk, Inc.
This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.
*/
'use strict';

const debuglog = __debug.bind(null, __filename);

var esprima = require('esprima');
var _ = require('underscore');
var fs = require('fs');
var validationConst = require('./constants').validationContants;
var manifestConfigs = require('../manifest');
var eventUtil = require('../utils/event-util');

const VALID_KEYS = ['event', 'callback'];

function validateEvents(evtHash) {
  var errMsgs = [];
  var eventsArr = [];
  var events = evtHash.events;
  events.forEach(function(ev) {
    var diff = _.difference(_.keys(ev), VALID_KEYS);
    if (diff.length > 0) {
      errMsgs.push(`Invalid key in events array: '${diff}'`);
    }
    var eventName = ev['event'];
    var callback = ev['callback'];
    if (_.contains(eventsArr, eventName)) {
      errMsgs.push(`Event '${eventName}' is already mapped to a callback`);
    }
    var product = Object.keys(manifestConfigs.product)[0];
    if (!eventUtil.isValidEvent(product, eventName)) {
      errMsgs.push(`Invalid event: '${eventName}'`);
    }
    else {
      eventsArr.push(eventName);
    }
    if (!_.contains(evtHash.definedFunctions, callback)) {
      errMsgs.push(`Event callback '${callback}' is not defined.`);
    }
  });

  return errMsgs;
}

function getDefinedFunctions(hash) {
  var definedFunc = [];
  _.each(hash, function(prop) {
    if (prop.value.type === 'FunctionExpression') {
      definedFunc.push(prop.key.name);
    }
  });
  return definedFunc;
}

function getEventsList() {
  var events = [];
  var definedFunc = [];
  var code = fs.readFileSync( `${process.cwd()}/server/server.js`, 'utf8');
  var ast = esprima.parse(code).body;
  var exportsExp = _.find(ast, function(node) {
    return (node.type === 'ExpressionStatement'
      && node.expression.type === 'AssignmentExpression'
      && node.expression.left.name === 'exports');
  });
  if (exportsExp) {
    var properties = exportsExp.expression.right.properties;
    definedFunc = getDefinedFunctions(properties);
    var eventProp = _.find(properties, function(prop) {
      return (prop.key.name === 'events'
        && prop.value.type === 'ArrayExpression');
    });
    var eventsHash = eventProp ? eventProp.value.elements : [];
    _.each(eventsHash, function(evt) {
      var values = evt.properties;
      var hash = {};
      _.each(values, function(p) {
         hash[p.key.name] = p.value.value;
      });
      events.push(hash);
    });
  }
  return {
    events: events,
    definedFunctions: definedFunc
  };
}

function isBackendApp() {
  return _.contains(manifestConfigs['features'], 'backend');
}

module.exports = {
  validate: function() {
    try {
      var errMsgs = [];
      if (isBackendApp()) {
        var evtHash = getEventsList();
        errMsgs  = validateEvents(evtHash);
      }
      return errMsgs;
    }
    catch(e) {}
  },

  validationType: [validationConst.PRE_PKG_VALIDATION, validationConst.RUN_VALIDATION]
}