'use strict';

const debuglog = __debug.bind(null, __filename);

const _ = require('underscore');
const fs = require('fs');
const url = require('url');
const path = require('path');
const linter = require('eslint').linter;
const esprima = require('esprima');
const fileUtil = require('../utils/file-util');
const manifest = require('../manifest');
const validationConst = require('./constants').validationContants;

const ALLOWED_EXTNS = [ '.js' ]; //TODO: Check with Raghu on adding HTML also
const BACKEND_DIRS = [ './server' ];
const FRONTEND_DIRS = [ './app' ];
const SKIP_FOLDERS = [ 'node_modules' ];
const ECMA6 = 6;

const rules = {
  'semi': 2,
  'no-debugger': 2,
  'no-extra-semi': 2,
  'no-extra-parens': 2,
  'curly': 2,
  'no-eval': 2,
  'no-alert': 2,
  'no-caller': 2,
  'no-process-env': 2
};

var messages = '';

/**
 *  This function is recursive; can be made tail recursive
 *  to make Geeth happy.
 */
function getNodes(predicate, node) {
  var result = []

  if (predicate(node)) {
    result.push(node);
  }

  Object.keys(node).forEach(key => {
    const child = node[key];

    if (typeof child === 'object' && child !== null) {
      if (Array.isArray(child)) {
        return child.forEach(e => {
          result = result.concat(getNodes(predicate, e));
        });
      }

      result = result.concat(getNodes(predicate, child));
    }
  });

  return result;
}

function getCallExpressionNodes(node) {
  function isCallExpression(node) {
    return node.type === 'CallExpression' &&
           node.callee.constructor.name.toString() === 'Identifier';
  }

  return getNodes(isCallExpression, node);
}

function getURLLiteralNodes(node) {
  function isURLLiteral(node) {
    const URLRegex = new RegExp('^(https?|ftp)://[^\s/$.?#].[^\s]*$', 'i');

    return node.type === 'Literal' &&
           typeof node.value === 'string' &&
           URLRegex.test(node.value);
  }

  return getNodes(isURLLiteral, node);
}

function showDeprecationMessages(nodes, file) {
  /**
   *  Key-Value pairs listing deprecated functions as keys with
   *  replacements as values.
   */
  const deprecations = {
    loadLib: 'require',
    loadDependency: 'require'
  };

  nodes.forEach(node => {
    const name = node.callee.name;

    if (deprecations.hasOwnProperty(name)) {
      console.log(`${file}::${node.loc.start.line}: '${name}' will soon be deprecated. Please use '${deprecations[name]}'`);
    }
  });
}

function checkDependencyUsage(functionCalls) {
  const messages = [];

  function isDependencyLoader(node) {
    return [
      'loadDependency',
      'require'
    ].includes(node.callee.name);
  }

  function isNPMDependency(node) {
    /**
     *  `require` can be used to load local files as well.
     *  So make sure we don't pick those as dependencies.
     */
    return !!node.arguments[0] &&
           !node.arguments[0].value.startsWith('.');
  }

  /**
   *  Horribly inefficient, but given we will only have
   *  ~10 elements, makes no measureable difference.
   */
  function isUnique(element, index, array) {
    return array.indexOf(element) === index;
  }

  const dependenciesSeen = functionCalls
                            .filter(isDependencyLoader)
                            .filter(isNPMDependency)
                            .map(e => e.arguments[0].value)
                            .filter(isUnique);

  const unusedDependencies = _.difference(Object.keys(manifest.dependencies), dependenciesSeen),
        unlistedDependencies = _.difference(dependenciesSeen, Object.keys(manifest.dependencies));

  if (unusedDependencies.length !== 0) {
    messages.push(`The following dependencies are not used: ${unusedDependencies}`);
  }

  if (unlistedDependencies.length !== 0) {
    messages.push(`The following dependencies are not listed in the manifest: ${unlistedDependencies}`);
  }

  return messages;
}

function printUnlistedURLs(nodes) {
  function isUnlisted(url) {
    return manifest
            .whitelistedDomains
            .every(listed => !(new RegExp(listed.replace('*', '[a-z0-9\-]*'))).test(url));
  }

  const unlisted = nodes
                    .map(node => node.value)
                    .filter(isUnlisted);

  if (unlisted.length) {
    console.log('Please ensure that the following URLs are listed in the manifest file.');
    console.log(unlisted.join('\n'));
  }
}

function lint(files) {
  var messages = [];
  var functionCallNodes = [];
  var urlNodes = [];

  files.forEach(file => {
    var fileContent = fileUtil.readFile(file);

    var lintMessages = linter.verify(fileContent, {
      rules: rules,
      allowInlineConfig: true,
      parserOptions: {
        ecmaVersion: ECMA6
      }
    });

    try {
      let ast = esprima.parse(fileContent, {
        loc: true
      });

      if(file.startsWith(`server${path.sep}`)) {
        let nodes = getCallExpressionNodes(ast);

        showDeprecationMessages(nodes, file);
        functionCallNodes = functionCallNodes.concat(nodes);
      }
      else if (file.startsWith(`app${path.sep}`)) {
        urlNodes = urlNodes.concat(getURLLiteralNodes(ast));
      }
    }
    catch (e) {
      console.log(e)
    }

    lintMessages.forEach(m => messages.push(`${file}::${m.line}: ${m.message}`));
  });

  printUnlistedURLs(urlNodes);

  messages = messages.concat(checkDependencyUsage(functionCallNodes));

  return messages;
}

module.exports = {
  validate: function() {
    var FILES = [];

    //Front end code for linting
    FRONTEND_DIRS.forEach(directory => {
      FILES = FILES.concat(fileUtil.spider(SKIP_FOLDERS, ALLOWED_EXTNS, directory));
    });

    //Backend code for linting
    if (_.contains(manifest.features, 'backend')) {
      BACKEND_DIRS.forEach(directory => {
        FILES = FILES.concat(fileUtil.spider(SKIP_FOLDERS, ALLOWED_EXTNS, directory));
      });
    }

    var messages = lint(FILES);
    var lintMessages = [];

    if (!(_.isEmpty(messages))) {
      lintMessages.push(messages);
    }

    //Return errors if there is any, else prompt for not using $request
    return lintMessages;
  },
  validationType: [validationConst.PRE_PKG_VALIDATION]
};
