'use strict';

const debuglog = __debug.bind(null, __filename);

var mf = require('../manifest');
var fs = require('fs');
var validationConst = require('./constants').validationContants;
var _ = require('underscore');
var fileUtil = require('../utils/file-util');
var OAUTH = "oauth";
var MANDATORY_KEYS = ['client_id', 'client_secret', 'authorize_url', 'token_url',
                      'token_type'];
var VALID_TOKEN_TYPES = ['account']
var VALID_KEYS = ["display_name", "description", "type", "required"];
var OAUTH_CONFIG_FILE = 'oauth_config.json';
var MANDATORY_OAUTH_IPARAM_KEYS = ['display_name', 'description', 'type'];
var VALID_KEYS = ["display_name", "description", "type", "required"];
var VALID_TYPES = ["text"];
var TYPE = "type";

function validateMandatoryKeys(oauthConfig) {
  var missingKeys = [];
  for (var key in oauthConfig) {
    if (oauthConfig[key] == null || oauthConfig[key] === "") {
      delete oauthConfig[key];
    }
  }
  missingKeys = _.difference(MANDATORY_KEYS, _.keys(oauthConfig))
  if (missingKeys.length > 0) {
    return `Mandatory key(s) missing in oauth_config.json - ${missingKeys.join(', ')}.`;
  }
}

function validateTokenType(oauthConfig) {
  if (!(_.isNull(oauthConfig.token_type)) && !(_.isUndefined(oauthConfig.token_type))) {
    if (!(_.contains(VALID_TOKEN_TYPES, oauthConfig.token_type))) {
      return `Invalid token type mentioned in oauth_config.json - ${oauthConfig.token_type}.`;
    }
  }
}

function validateOauthIparams(oauthConfig) {
  var oauthIparams = oauthConfig.oauth_iparams;
  if (oauthIparams) {
    var errs = [];
    var keywordsUsed = _.intersection(VALID_KEYS, Object.keys(oauthIparams));
    if (keywordsUsed.length > 0) {
      return `Reserved keywords ${keywordsUsed} used as keys in ${OAUTH_CONFIG_FILE}.`;
    }
    for (var key in oauthIparams) {
      for (var detail in oauthIparams[key]) {
        if (oauthIparams[key][detail] && _.isEmpty(oauthIparams[key][detail])) {
          delete oauthIparams[key][detail];
        }
      }
      if (key.length === 0) {
        errs.push('The oauth_iparams key should be a non-empty string.');
        continue;
      }
      var missingKeys = _.difference(MANDATORY_OAUTH_IPARAM_KEYS, _.keys(oauthIparams[key]));
      if (!(_.isObject(oauthIparams[key])) || missingKeys.length > 0) {
        errs.push(
          `Mandatory key(s) ${missingKeys} missing for ${key} in ${OAUTH_CONFIG_FILE}.`
        );
      }
      if (!_.contains(VALID_TYPES, oauthIparams[key][TYPE])) {
        errs.push(
          `Invalid type '${oauthIparams[key][TYPE]}' found in ${OAUTH_CONFIG_FILE}.`);
      }
      if (_.difference(Object.keys(oauthIparams[key]), VALID_KEYS).length > 0) {
        errs.push(`Invalid keys in ${OAUTH_CONFIG_FILE}`);
      }
    }
    return errs;
  }
}

module.exports = {
  validate: function() {
    var errMsgs = [];
    var oauthConfig;
    if (_.contains(mf.features, OAUTH)) {
      try {
        oauthConfig = JSON.parse(fileUtil.readFile('./config/oauth_config.json'));
      }
      catch (e) {
        return ["oauth_config.json - " + e.message];
      }
      [ validateMandatoryKeys,
        validateTokenType,
        validateOauthIparams ].forEach(function(validator) {
        var validationMessage = validator(oauthConfig);
        if (!(_.isUndefined(validationMessage))) {
          errMsgs.push(validationMessage);
        }
      });
    }
    return errMsgs;
  },

  validationType: [validationConst.PRE_PKG_VALIDATION, validationConst.RUN_VALIDATION]
}
