var expect = require('chai').expect;
var tmp = require('tmp');
var fs = require('fs-extra');

describe('Backend $request test', function () {

  var projDir, $request, RequestAPI;

  this.timeout(50000);

  before(function() {
    projDir = tmp.dirSync({prefix: 'freshapps_sdk'});
    require(__dirname + '/../lib/cli/init').run(projDir['name'], 'your_first_app');
    process.chdir(projDir['name']);
    RequestAPI = require('../lib/event_handler/features/request');
    $request = new RequestAPI;
  });

  it('OAuth refresh', function(done) {
    $request.get('https://httpbin.org/status/401', { isOAuth: true})
    .then(function(data) {
      done();
    }, function(err) {
      expect(err.code).to.eql('ECONNREFUSED');
      done();
    });
  })

  it('Unsupported content type', function(done) {
    $request.get('https://httpbin.org/image/png', {})
    .then(function(data) {
      done();
    }, function(err) {
      expect(err).to.eql({ status: 415,
        headers: {},
        errorSource: "APP",
        response: 'Unsupported content type' });
      done();
    })
  })

  it('make request', function(done) {
    $request.get('https://httpbin.org/get')
    .then(function(data) {
      expect(data.status).to.eql(200);
      done();
    }, function(err) {
      done();
    })
  })

  it('contains all method', function(done) {
    expect(Object.keys($request)).to.eql([ 'get', 'post', 'put', 'patch', 'delete' ]);
    done();
  })
});

describe('Schedule event test', function() {
  var ScheduleAPI, $schedule, projDir, server;

  this.timeout(50000);

  before(function() {
    projDir = tmp.dirSync({prefix: 'freshapps_sdk'});
    require(__dirname + '/../lib/cli/init').run(projDir['name'], 'your_first_serverless_app');
    process.chdir(projDir['name']);
    var testServerFile = __dirname + '/../test-res/backend_server.js';
    var serverFile = projDir['name'] + '/server/server.js';
    require(__dirname + '/../lib/manifest').reload();
    fs.copySync(testServerFile, serverFile);
    delete require.cache;
    fs.writeFileSync(projDir['name'] + '/.report.json', JSON.stringify({}));
    server = require('../lib/cli/run').run();

    ScheduleAPI = require('../lib/event_handler/features/schedule');
    $schedule = new ScheduleAPI({
      svrExecScript: {
        events: [
          {event: 'onScheduledEvent', callback: 'onScheduledEvent'}
        ]
      }
    });
  });

  after(function() {
    server.close();
  });

  it('Error out - Mandatory Keys missing', function(done) {
    $schedule.create({
      name: '1234'
    })
    .then(null, function(err) {
      expect(err.status).to.eql(400);
      expect(err.message).to.eql('The following mandatory keys are missing - data,schedule_at');
      done();
    });
  });

  it('Error out - Invalid data type', function(done) {
    $schedule.create({
      name: '1234',
      data: 'hello',
      schedule_at: '2019-02-20T17:38:38.626Z'
    })
    .then(null, function(err) {
      expect(err.status).to.eql(400);
      expect(err.message).to.eql(`The 'data' value must be of type JSON`);
      done();
    });
  });

  it('Error out - Invalid start time (not in ISO)', function(done) {
    $schedule.create({
      name: '1234',
      data: {'hello': 'world'},
      schedule_at: '2019-xy-ty'
    })
    .then(null, function(err) {
      expect(err.status).to.eql(400);
      expect(err.message).to.eql(`The 'schedule_at' value must be an ISO string`);
      done();
    });
  });

  it('Error out - Missing keys in repeat section', function(done) {
    $schedule.create({
      name: '1234',
      data: {'hello': 'world'},
      schedule_at: '2019-02-20T17:38:38.626Z',
      repeat: {

      }
    })
    .then(null, function(err) {
      expect(err.status).to.eql(400);
      expect(err.message).to.eql('The following mandatory keys are missing - time_unit,frequency');
      done();
    });
  });

  it('Error out - Wrong repeat time unit', function(done) {
    $schedule.create({
      name: '1234',
      data: {'hello': 'world'},
      schedule_at: '2019-02-20T17:38:38.626Z',
      repeat: {
        time_unit: 'decade',
        frequency: 1,
        timezone: 'Asia/Kolkata'
      }
    })
    .then(null, function(err) {
      expect(err.status).to.eql(400);
      expect(err.message).to.eql('Time unit must either be days, minutes or hours');
      done();
    });
  });

  it('Error out - Invalid repeat frequency', function(done) {
    $schedule.create({
      name: '1234',
      data: {'hello': 'world'},
      schedule_at: '2019-02-20T17:38:38.626Z',
      repeat: {
        time_unit: 'minutes',
        frequency: 1000,
        timezone: 'Asia/Kolkata'
      }
    })
    .then(null, function(err) {
      expect(err.status).to.eql(400);
      expect(err.message).to.eql(`The maximum frequency value for the 'minutes' parameter is 60`);
      done();
    });
  });

  it('Error out - Invalid timezone', function(done) {
    $schedule.create({
      name: '1234',
      data: {'hello': 'world'},
      schedule_at: '2019-02-20T17:38:38.626Z',
      repeat: {
        time_unit: 'minutes',
        frequency: 30,
        timezone: 'Mars Timezone'
      }
    })
    .then(null, function(err) {
      expect(err.status).to.eql(400);
      expect(err.message).to.eql('Invalid timezone');
      done();
    });
  });

  it('Create one time schedule', function(done) {
    $schedule.create({
      name: '1234',
      data: {'hello': 'world'},
      schedule_at: '2019-02-20T17:38:38.626Z'
    })
    .then(function(success) {
      expect(success.status).to.eql(200);
      setTimeout(function() {
        done();
      }, 4000)
    });
  });

  it('Create recurring schedule', function(done) {
    $schedule.create({
      name: '1234',
      data: {'hello': 'world'},
      schedule_at: '2019-02-20T17:38:38.626Z',
      repeat: {
        time_unit: 'hours',
        frequency: 3,
        timezone: 'Asia/Kolkata'
      }
    })
    .then(function(success) {
      expect(success.status).to.eql(200);
      setTimeout(function() {
        done();
      }, 4000);
    });
  });

  it('Fetch schedule', function(done) {
    $schedule.create({
      name: '123456',
      data: {'hello': 'world'},
      schedule_at: '2019-02-20T17:38:38.626Z'
    })
    .then(function(success) {
      setTimeout(function() {
        $schedule.fetch({
          name: '123456'
        })
        .then(function(data) {
          expect(data.name).to.eql('123456');
          done();
        });
      }, 4000);
    });
  });

  it('Delete schedule', function(done) {
    $schedule.create({
      name: '12345678',
      data: {'hello': 'world'},
      schedule_at: '2019-02-20T17:38:38.626Z'
    })
    .then(function(success) {
      $schedule.delete({
        name: '12345678'
      })
      .then(function(data) {
        expect(data.status).to.eql(200);
        setTimeout(function() {
          done();
        }, 4000)
      });
    });
  });

})
