var tmp = require('tmp');
var expect = require('chai').expect;
var fs = require('fs-extra');
var iparamValidator;
var request = require('supertest');

describe('iparam validate', function() {

  var projDir, destIparamFile, destTestDataFile, destIparamHtmlFile;

  before(function() {
    projDir = tmp.dirSync({prefix: 'freshapps_sdk'});
    process.chdir(projDir.name);
    require(__dirname + '/../lib/cli/init').run(projDir['name'], 'your_first_app');
    process.chdir(projDir['name']);
    require(__dirname + '/../lib/manifest').reload();
    iparamValidator = require('../lib/validations/iparam-validation')
    destIparamFile = projDir['name'] + '/config/iparams.json';
    destTestDataFile = projDir['name'] + '/config/iparam_test_data.json';
    destIparamHtmlFile = projDir['name'] + '/config/iparams.html';
  });

  it('should validate keys', function(done) {
    expect([]).eql(iparamValidator.validate());
    done();
  });

  it('should fail', function(done) {
    var invalidIparamFile = __dirname + '/../test-res/iparam/invalid_iparam.json';
    var validIparamFile = __dirname + '/../test-res/iparam/valid_iparam.json';
    fs.copySync(invalidIparamFile, destIparamFile);
    var result = ["Invalid type 'textd' found in iparams.json.",
      "Invalid value specified for 'required' for 'name' in iparams.json. It should be a boolean.",
      "Error message is missing for the regex 'test' for 'name' in iparams.json.",
      "Invalid regex provided for 'test' for 'name' in iparams.json.",
      "Regex should not be empty in iparams.json for 'email'.",
      "For 'default_value' of the 'email': Please enter a valid email.",
      "Invalid value specified for 'default_value' for 'checkbox' in iparams.json. For checkbox, it should be a boolean.",
    ];
    expect(result).eql(iparamValidator.validate());
    fs.copySync(validIparamFile, destIparamFile);
    done();
  });

  it('should validate files when iparams.html exists', function(done) {
    var validIparamFile = __dirname + '/../test-res/iparam/valid_iparam.json';
    var testDataFile = __dirname + '/../test-res/iparam/valid_iparam_test_data.json';
    var iparamHtmlFile = __dirname + '/../test-res/iparam/valid_iparams.html';
    fs.copySync(testDataFile, destTestDataFile);
    fs.copySync(iparamHtmlFile, destIparamHtmlFile);
    fs.copySync(validIparamFile, destIparamFile);
    expect(["Unsupported File(s). Specify either iparams.html or iparams.json"]).eql(iparamValidator.validate());
    done();
  });

  it('should validate iparams.html file', function(done) {
    fs.removeSync(destIparamFile);
    fs.removeSync(destIparamHtmlFile);
    var testDataFile = __dirname + '/../test-res/iparam/valid_iparam_test_data.json';
    var iparamHtmlFile = __dirname + '/../test-res/iparam/invalid_iparams.html';
    fs.copySync(testDataFile, destTestDataFile);
    fs.copySync(iparamHtmlFile, destIparamHtmlFile);
    expect(["Mandatory method(s) missing in iparams.html: getConfigs,postConfigs."]).eql(iparamValidator.validate());
    iparamHtmlFile = __dirname + '/../test-res/iparam/invalid_iparam1.html';
    fs.copySync(iparamHtmlFile, destIparamHtmlFile);
    expect(["Mandatory method(s) missing in iparams.html: postConfigs."]).eql(iparamValidator.validate());
    done();
  });

  it('evaluate custom installation page endpoints', function(done) {
    fs.removeSync(destIparamFile);
    fs.removeSync(destIparamHtmlFile);
    var iparamHtmlFile = __dirname + '/../test-res/iparam/custom_iparams.html';
    fs.copySync(iparamHtmlFile, destIparamHtmlFile);
    var server = require('../lib/cli/run').run();
    request(server)

    /*
      Fetch configuration page - must return 200
    */
    .get('/custom_configs')
    .end(function(e, r) {
      expect(r.status).to.eql(200);
      request(server)

      /*
        Fetch user iparams.html (form) - must return config/iparams.html
      */
      .get('/custom_configs/form')
      .end(function(e, r) {
        expect(r.text).to.eql(fs.readFileSync(`${projDir.name}/config/iparams.html`).toString());
        request(server)

        /*
          Fetching custom installation parameters - must return empty object({}), since nothing stored
        */
        .get('/custom_configs/fetch')
        .end(function(e, r) {
          expect(r.text).to.eql('{"isCustomIParam":true,"customIParams":{}}')
          request(server)

          /*
            Store custom installation parameters
          */
          .post('/custom_configs/store')
          .send({ 'hello': 'world' })
          .end(function(e, r) {
            expect(r.status).to.eql(200);
            request(server)

            /*
              Fetch the stored parameters - must be equal
            */
            .get('/custom_configs/fetch')
            .end(function(e, r) {
              expect(r.text).to.eql('{"isCustomIParam":true,"customIParams":{"hello":"world"}}');
              done();
              server.close();
            })
          })
        })
      })
    })
  });

  it('check custom installation endpoints for oauth app', function(done) {
    var oauthConfigFile = __dirname + '/../test-res/iparam/oauth_config.json';
    var oauthConfigDestFile = `${projDir.name}/config/oauth_config.json`;
    fs.copySync(oauthConfigFile, oauthConfigDestFile);
    require(__dirname + '/../lib/manifest').reload();
    var server = require('../lib/cli/run').run();
    request(server)

    /*
      Get request for custom installation page must return 302 if its oauth app (& is not authorized)
    */
    .get('/custom_configs')
    .end(function(e, r) {
      expect(r.status).to.eql(302);
      fs.remove(`${projDir.name}/config/iparams.html`)
      request(server)

      /*
        Request to fetch form must return 404 if iparams.html doesn't exist in the project
      */
      .get('/custom_configs/form')
      .end(function(e, r) {
        expect(r.status).to.eql(200);
        done();
        server.close();
      })
    })

  })

});
