/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/
var tmp = require('tmp');
var expect = require('chai').expect;
var fs = require('fs-extra');

describe('manifest validate', function() {

  var projDir, mf, mfValidate, mfDependencyValidate;

  before(function() {
    projDir = tmp.dirSync({prefix: 'freshapps_sdk'});
    require(__dirname + '/../lib/cli/init').run(projDir['name'], 'your_first_app');
    process.chdir(projDir['name']);
    mf = require(__dirname + '/../lib/manifest');
    mfValidate = require(__dirname+
      '/../lib/validations/manifest-validation');
    mfDependencyValidate = require(__dirname +
      '/../lib/validations/manifest-dependency-validation');
  });

  it('should fail - invalid domain', function(done) {
    var srcmfFile = __dirname + '/../test-res/manifest/manifest_invalid_domain.json';
    var validsrcmFile = __dirname + '/../test-res/manifest/manifest_valid.json';
    var destmfFile = projDir['name'] + '/manifest.json';
    fs.copySync(srcmfFile, destmfFile);
    mf.reload();
    var validationErrors = mfValidate.validate();
    expect(validationErrors).to.include('Whitelisted domains must not have path: https://www.abc.com/test');
    expect(validationErrors).to.include('Whitelisted domains must not end with a \'/\': https://abc.com/');
    expect(validationErrors).to.include('Whitelisted domains must use HTTPS: http://www.abc.com');
    expect(validationErrors).to.include('Whitelisted domains must not be localhost: https://localhost');
    expect(validationErrors).to.include('Whitelisted domains must not contain IP addresses: https://127.0.0.1')
    expect(validationErrors).to.include('Whitelisted domains must not have more than one subdomain: https://*.*.freshdesk.com');
    fs.copySync(validsrcmFile, destmfFile);
    mf.reload();
    done();
  });

  it('should fail - invalid dependency', function(done) {
    var srcmfFile = __dirname + '/../test-res/manifest/manifest_invalid_dependency.json';
    var validsrcmFile = __dirname + '/../test-res/manifest/manifest_valid.json';
    var destmfFile = projDir['name'] + '/manifest.json';
    fs.copySync(srcmfFile, destmfFile);
    mf.reload();
    expect(mfDependencyValidate.validate()[0]).eql('Unknown dependency mentioned in manifest.json: abcxrt.');
    fs.copySync(validsrcmFile, destmfFile);
    mf.reload();
    done();
  });

  it('should fail - url is empty', function(done) {
    var srcmfFile = __dirname + '/../test-res/manifest/manifest_no_url.json';
    var validsrcmFile = __dirname + '/../test-res/manifest/manifest_valid.json';
    var destmfFile = projDir['name'] + '/manifest.json';
    fs.copySync(srcmfFile, destmfFile);
    mf.reload();
    expect(mfValidate.validate()[0]).eql('Url is either not mentioned or empty in freshdesk/ticket_sidebar');
    fs.copySync(validsrcmFile, destmfFile);
    mf.reload();
    done();
  });

  it('should fail - url is not in appfolder', function(done) {
    var srcmfFile = __dirname + '/../test-res/manifest/manifest_missing_template.json';
    var validsrcmFile = __dirname + '/../test-res/manifest/manifest_valid.json';
    var destmfFile = projDir['name'] + '/manifest.json';
    fs.copySync(srcmfFile, destmfFile);
    mf.reload();
    expect(mfValidate.validate()[0]).eql(`Template file 'abc.html' mentioned in freshdesk/ticket_sidebar is not found in app folder`);
    fs.copySync(validsrcmFile, destmfFile);
    mf.reload();
    done();
  });

  it('should fail - icon is empty', function(done) {
    var srcmfFile = __dirname + '/../test-res/manifest/manifest_no_icon.json';
    var validsrcmFile = __dirname + '/../test-res/manifest/manifest_valid.json';
    var destmfFile = projDir['name'] + '/manifest.json';
    fs.copySync(srcmfFile, destmfFile);
    mf.reload();
    expect(mfValidate.validate()[0]).eql('Icon is either not mentioned or empty in freshdesk/ticket_sidebar');
    fs.copySync(validsrcmFile, destmfFile);
    mf.reload();
    done();
  });

  it('should fail - icon is not in appfolder', function(done) {
    var srcmfFile = __dirname + '/../test-res/manifest/manifest_missing_icon.json';
    var validsrcmFile = __dirname + '/../test-res/manifest/manifest_valid.json';
    var destmfFile = projDir['name'] + '/manifest.json';
    fs.copySync(srcmfFile, destmfFile);
    mf.reload();
    expect(mfValidate.validate()[0]).eql(`Icon 'logo.jpg' mentioned in freshdesk/ticket_sidebar is not found in app folder`);
    fs.copySync(validsrcmFile, destmfFile);
    mf.reload();
    done();
  });

  it('should fail - invalid icon dimensions', function(done) {
    var srcmfFile = __dirname + '/../test-res/logo_test.png';
    var validsrcmFile = __dirname + '/../test-res/logo.png';
    var destmfFile = projDir['name'] + '/app/logo.png';
    fs.copySync(srcmfFile, destmfFile);
    mf.reload();
    expect(mfValidate.validate()[0]).eql(`Invalid dimension of icon 'logo.png' for freshdesk/ticket_sidebar`);
    fs.copySync(validsrcmFile, destmfFile);
    mf.reload();
    done();
  });

  it('should fail - product is empty', function(done) {
    var srcmfFile = __dirname + '/../test-res/manifest/manifest_no_product.json';
    var validsrcmFile = __dirname + '/../test-res/manifest/manifest_valid.json';
    var destmfFile = projDir['name'] + '/manifest.json';
    fs.copySync(srcmfFile, destmfFile);
    mf.reload();
    expect(mfValidate.validate()[0]).eql(`Atleast one product must be mentioned in manifest.json`);
    fs.copySync(validsrcmFile, destmfFile);
    mf.reload();
    done();
  });

  it('should fail - manifest not found', function(done) {
    var manifestFile = projDir['name'] + '/manifest.json';
    fs.unlinkSync(manifestFile);
    mf.reload();
    done();
  });

  it('should fail - unable to parse manifest', function(done) {
    var manifestFile = projDir['name'] + '/manifest.json';
    fs.writeFileSync(manifestFile, "sometext");
    mf.reload();
    done();
  });
});
