var expect = require('chai').expect;
var tmp = require('tmp');
var fs = require('fs-extra');
var request = require('supertest');
var passportRefresh = require('passport-oauth2-refresh');
var sinon = require('sinon');

describe('Oauth Test', function() {
  var server, projDir;

  before(function() {
    projDir = tmp.dirSync({prefix: 'freshapps_sdk'});
    require(__dirname + '/../lib/cli/init').run(projDir['name'], 'your_first_app');
    process.chdir(projDir['name']);
    validator = require('../lib/validations/oauth-validation');
  });

  afterEach(function() {
    try {
      server.close();
      projDir.removeCallback();
    }
    catch(e) {}
  })

  it('should fail - invalid oauth config', function(done) {
    var invalidOauthConfigFile = __dirname + '/../test-res/iparam/invalid_oauth_config.json';
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(invalidOauthConfigFile, oauthConfigFile);
    require(__dirname + '/../lib/manifest').reload();
    expect(validator.validate()).eql(['Mandatory key(s) missing in oauth_config.json - token_type.']);
    done();
  });

  it('should fail - invalid oauth config - invalid JSON', function(done) {
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.writeFileSync(oauthConfigFile, '{');
    require(__dirname + '/../lib/manifest').reload();
    expect(validator.validate()).eql(['oauth_config.json - Unexpected end of JSON input']);
    done();
  });

  it('should fail - invalid oauth iparam - type', function(done) {
    var invalidOauthConfigFile = __dirname + '/../test-res/iparam/invalid_oauth_iparam-type.json';
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(invalidOauthConfigFile, oauthConfigFile);
    require(__dirname + '/../lib/manifest').reload();
    expect(validator.validate()).eql([["Invalid type 'integer' found in oauth_config.json."]]);
    done();
  });

  it('should fail - invalid oauth iparam - reserved key', function(done) {
    var invalidOauthConfigFile = __dirname + '/../test-res/iparam/invalid_oauth_iparam-reserved-key.json';
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(invalidOauthConfigFile, oauthConfigFile);
    require(__dirname + '/../lib/manifest').reload();
    expect(validator.validate()).eql(["Reserved keywords display_name used as keys in oauth_config.json."]);
    done();
  });

  it('should fail - invalid oauth iparam - invalid key', function(done) {
    var invalidOauthConfigFile = __dirname + '/../test-res/iparam/invalid_oauth_iparam-reserved-key.json';
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(invalidOauthConfigFile, oauthConfigFile);
    require(__dirname + '/../lib/manifest').reload();
    expect(validator.validate()).eql(["Reserved keywords display_name used as keys in oauth_config.json."]);
    done();
  });

  it('should fail - invalid oauth iparam - empty key', function(done) {
    var invalidOauthConfigFile = __dirname + '/../test-res/iparam/invalid_oauth_iparam-empty-key.json';
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(invalidOauthConfigFile, oauthConfigFile);
    require(__dirname + '/../lib/manifest').reload();
    expect(validator.validate()).eql([ [ 'The oauth_iparams key should be a non-empty string.' ] ]);
    done();
  });

  it('should fail - invalid oauth iparam - missing mandatory keys', function(done) {
    var invalidOauthConfigFile = __dirname + '/../test-res/iparam/invalid_oauth_iparam-missing-mandatory-key.json';
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(invalidOauthConfigFile, oauthConfigFile);
    require(__dirname + '/../lib/manifest').reload();
    expect(validator.validate()).eql([[ "Mandatory key(s) display_name missing for subdomain in oauth_config.json." ]]);
    done();
  });

  it('should fail - invalid oauth iparam - invalid additional keys', function(done) {
    var invalidOauthConfigFile = __dirname + '/../test-res/iparam/invalid_oauth_iparam-invalid-key.json';
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(invalidOauthConfigFile, oauthConfigFile);
    require(__dirname + '/../lib/manifest').reload();
    expect(validator.validate()).eql([[ "Invalid keys in oauth_config.json" ]]);
    done();
  });

  it('should succeed - Oauth auth index', function(done) {
    var validOauthConfigFile = __dirname + '/../test-res/iparam/oauth_config.json';
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(validOauthConfigFile, oauthConfigFile);
    server = require('../lib/cli/run').run();
    request(server)
      .get('/auth/index?callback=dummy')
      .send()
      .end(function(a, b) {
        expect(b.status).to.eql(302);
        request(server)
          .get('/accesstoken')
          .end(function(a, b) {
            expect(b.status).to.eql(200);
            request(server)
              .get('/iframe/api')
              .end(function(a, b) {
                expect(b.status).to.eql(200);
                done();
              });
          });
      });
  });

  it('should succeed - Oauth auth callback', function(done) {
    var validOauthConfigFile = __dirname + '/../test-res/iparam/oauth_config.json';
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(validOauthConfigFile, oauthConfigFile);
    server = require('../lib/cli/run').run();
    request(server)
      .get('/auth/callback?callback=dummy')
      .send()
      .expect(200, done);
  })

  it('should succeed - Fetching oauth iparams', function(done) {
    var validOauthConfigFile = __dirname + '/../test-res/iparam/oauth_config_iparam.json';
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(validOauthConfigFile, oauthConfigFile);
    server = require('../lib/cli/run').run();
    request(server)
      .get('/oauth_iparams')
      .send()
      .expect(200, '{"subdomain":{"display_name":"subdomain","description":"subdomain","type":"text","required":true}}', done);
  })

  it('should succeed - Fetching oauth iparams - not specified', function(done) {
    var validOauthConfigFile = __dirname + '/../test-res/iparam/oauth_config.json';
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(validOauthConfigFile, oauthConfigFile);
    server = require('../lib/cli/run').run();
    request(server)
      .get('/oauth_iparams')
      .send()
      .expect(200, '{}', done);
  })

  it('should succeed - Oauth refresh', function(done) {
    var validOauthConfigFile = __dirname + '/../test-res/iparam/oauth_config.json';
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(validOauthConfigFile, oauthConfigFile);
    server = require('../lib/cli/run').run();
    request(server)
      .post('/dprouter')
      .send({action: 'refresh'})
      .set({
        "content-type":"application/json",
        "MKP-ROUTE": "oauth"
      })
      .expect(302, done);
  })

  it('should fail - Oauth refresh', function(done) {
    passportRefresh.requestNewAccessToken.restore();
    sinon.stub(passportRefresh, 'requestNewAccessToken').callsFake(function(a, b, c) {
      c("sdf");
    })
    var validOauthConfigFile = __dirname + '/../test-res/iparam/oauth_config.json';
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(validOauthConfigFile, oauthConfigFile);
    server = require('../lib/cli/run').run();
    request(server)
      .post('/dprouter')
      .send({action: 'refresh'})
      .set({
        "content-type":"application/json",
        "MKP-ROUTE": "oauth"
      })
      .expect(302, done);
  })

})
