var tmp = require('tmp');
var request = require('supertest');
var fs = require('fs-extra');
var proxymFile = __dirname + '/../test-res/manifest/manifest_proxy.json';
var expect = require('chai').expect;
var storage = require('../lib/utils/data-store');
var nsUtil = require('../lib/utils/ns-resolver');
var nock = require('nock');

var keyNamespace = function() {
  return `${nsUtil.getNamespace()['app_id']}_oauth`;
}

describe('Proxy test', function () {
  this.timeout(15000);
  var projDir, server, destmfFile;

  before(function() {
    projDir = tmp.dirSync({prefix: 'freshapps_sdk'});
    require(__dirname + '/../lib/cli/init').run(projDir['name'], 'your_first_app');
    process.chdir(projDir['name']);
    destmfFile = projDir['name'] + '/manifest.json';
    fs.copySync(proxymFile, destmfFile);
    require(__dirname + '/../lib/manifest').reload();
  })

  beforeEach(function () {
    server = require('../lib/cli/run').run();
  });

  afterEach(function () {
    server.close();
  });

  it('should make request', function(done) {
    var validOauthConfigFile = __dirname + '/../test-res/iparam/oauth_config.json';
    var oauthConfigFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(validOauthConfigFile, oauthConfigFile);
    require(__dirname + '/../lib/manifest').reload();
    storage.store(keyNamespace(), "access token");
    var payload = {
      "action": "execute",
      "data":
      {
        "method": "get",
        "url": "https://www.google.com"
      }
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "MKP-ROUTE": "proxy"
      })
      .expect(200,done);
  });

  it('should fail request - invalid substitution', function(done) {
    var payload = {
      "action": "execute",
      "data":
      {
        "method": "get",
        "url": "https://www.google.com",
        "headers": { "Authorization" : "<%= y.er %>"}
      }
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "MKP-ROUTE": "proxy"
      })
      .expect(200,done);
  });

  it('should succeed request - add err source', function(done) {
    var payload = {
      "action": "execute",
      "data":
      {
        "method": "get",
        "url": "https://httpbin.org/status/400"
      }
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "MKP-ROUTE": "proxy"
      })
      .end(function(err, res) {
        expect(res.body.errorSource).to.eql("APP");
        done();
      })
  });

  it('should succeed request - random route', function(done) {
    var payload = {
      "action": "execute",
      "data":
      {
        "method": "get",
        "url": "https://abcdefghijklok.abcdefghijkl.kiolp/"
      }
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "MKP-ROUTE": "proxy"
      })
      .expect(200,done);
  });

  it('should fail request - Domain not whitelisted', function(done) {
    var payload = {
      "action": "execute",
      "data":
      {
        "method": "get",
        "url": "https://www.abc.com"
      }
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "MKP-ROUTE": "proxy"
      })
      .end(function(err, res) {
        expect(res.statusCode).to.equal(200);
        expect(res.text).to.equal('{"status":400,"headers":{},"response":"This domain has not been whitelisted","errorSource":"APP"}');
        done();
      })
  });

  it('should fail request - Domain not fully qualified', function(done) {
    var payload = {
      "action": "execute",
      "data":
      {
        "method": "get",
        "url": "https://www._abc.com"
      }
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "MKP-ROUTE": "proxy"
      })
      .end(function(err, res) {
        expect(res.statusCode).to.equal(200);
        expect(res.text).to.equal('{"status":400,"headers":{},"response":"This domain has not been whitelisted","errorSource":"APP"}');
        done();
      })
  });

  it('should fail request - unsupported content type', function(done) {
    var payload = {
      "action": "execute",
      "data":
      {
        "method": "get",
        "url": "https://www.google.com"
      }
    };
    nock('https://www.google.com').get('/').reply(200, {}, {
      'content-type': 'application/text'
    });
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({ "MKP-ROUTE": "proxy" })
      .end(function(err, resp) {
        expect(resp.statusCode).to.equal(200);
        expect(resp.text).to.equal('{"status":415,"headers":{},"response":"Unsupported content type","errorSource":"APP"}');
        done();
      });
  });

});
